/*
 * Copyright (C) 2003-2004 Imendio AB
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
 
#include <glib.h>
#include <string.h>
#include <stdlib.h>
#include <loudmouth/loudmouth.h>
#ifdef __WIN32__
#include <winsock2.h>
#endif
 
typedef struct {
    gchar	*name;
    gchar	*passwd;
} UserInfo;

typedef struct {
    GMainLoop	*main_loop;
} msg_data_t;
 
static void
free_user_info (UserInfo *info)
{
    g_free (info->name);
    g_free (info->passwd);
 
    g_free (info);
}
 
static void
authentication_cb (LmConnection *connection, gboolean result, gpointer ud)
{
    gchar *string;

    g_print ("Auth: %d\n", result);
    free_user_info ((UserInfo *) ud);
 
    if (result == TRUE) {
	LmMessage *m;
                 
	m = lm_message_new_with_sub_type (NULL,
					  LM_MESSAGE_TYPE_PRESENCE,
					  LM_MESSAGE_SUB_TYPE_AVAILABLE);
	string = lm_message_node_to_string (m->node);
	g_print (":: %s\n", string);
	free (string);
                 
	lm_connection_send (connection, m, NULL);
	lm_message_unref (m);
    }

}
 
static void
connection_open_cb (LmConnection *connection, gboolean result, UserInfo *info)
{
    g_print ("Connected callback\n");
    lm_connection_authenticate (connection,
				info->name, info->passwd, "TestLM",
				authentication_cb, info, FALSE,  NULL);
    g_print ("Sent auth message\n");
}

static void
send_reply (LmConnection	*connection,
	    const char		*peer,
	    const char		*message)
{
    LmMessage *reply;
    gboolean result;
    GError *error = NULL;

    reply = lm_message_new (peer, LM_MESSAGE_TYPE_MESSAGE);

    lm_message_node_add_child (reply->node, "body", message);

    result = lm_connection_send (connection, reply, &error);
    lm_message_unref (reply);

    if (! result) {
	g_error ("lm_connection_send failed");
    }
}

static void
handle_command (LmConnection	*connection,
		const char	*peer,
		const char	*command,
		gpointer	 closure)
{
    msg_data_t *data = closure;
    char *error;

    if (strcmp (command, "quit") == 0) {
	g_main_loop_quit (data->main_loop);
	return;
    }

    error = g_strdup_printf ("Unknown command: '%s'", command);
    send_reply (connection, peer, error);
    free (error);
}
 
static LmHandlerResult
handle_messages (LmMessageHandler *handler,
                 LmConnection     *connection,
                 LmMessage        *m,
                 gpointer          closure)
{
    LmMessageNode *body;
    const char *peer;
    const char *body_str;

    peer = lm_message_node_get_attribute (m->node, "from");


    g_print ("Incoming message from: %s\n", peer);

    body = lm_message_node_get_child (m->node, "body");
    if (body) {
	body_str = lm_message_node_get_value (body);

	if (body_str && body_str[0] == '%')
	    handle_command (connection, peer, body_str + 1, closure);
	else
	    send_reply (connection, peer, body_str);
    }
	
    return LM_HANDLER_RESULT_REMOVE_MESSAGE;
}
int
main (int argc, char **argv)
{
    LmConnection     *connection;
    LmMessageHandler *handler;
    gboolean          result;
    UserInfo         *info;
    gchar            *jid;
    msg_data_t	      data;
    GError	     *error;

    if (argc < 6) {
	g_print ("Usage: %s <server> <username> <password> <connectserver> <connectport>\n", argv[0]);
	return 1;
    }
                                                                                
    connection = lm_connection_new (argv[4]);

    jid = g_strdup_printf ("%s@%s", argv[2], argv[1]);
    lm_connection_set_jid (connection, jid);
    g_free (jid);

    lm_connection_set_port (connection, strtol (argv[5], (char **) NULL, 10));

    data.main_loop = g_main_loop_new (NULL, FALSE);

    handler = lm_message_handler_new (handle_messages, &data, NULL);
    lm_connection_register_message_handler (connection, handler,
					    LM_MESSAGE_TYPE_MESSAGE,
					    LM_HANDLER_PRIORITY_NORMAL);
                                                                                
    lm_message_handler_unref (handler);
                                                                                
    info = g_new0 (UserInfo, 1);
    info->name = g_strdup (argv[2]);
    info->passwd = g_strdup (argv[3]);
                                                                                
    result = lm_connection_open (connection,
				 (LmResultFunction) connection_open_cb,
				 info, NULL, NULL);
                                                                                
    if (!result) {
	g_print ("Opening connection failed: %d\n", result);
    } else {
	g_print ("Returned from the connection_open\n");
    }
                                                                                
    g_main_loop_run (data.main_loop);

    g_main_loop_unref (data.main_loop);

    lm_connection_close (connection, &error);
    lm_connection_unref (connection);
                                                                                
    return 0;
}
